package iBus.layers;

import iBus.*;
import iBus.exception.*;

/**
 * <P>TEMPLATE: Template for new iBus protocol objects. <BR>
 * NOT TO BE USED IN PROTOCOL STACKS.
 *
 * <P>(give a short description of the main purpose of the object)<BR>
 *
 * <P><STRONG>PARAMETERS:</STRONG>
 * <UL>
 * <LI> 
 *    testparam: int: >= 0. Here you describe the purpose of the parameter.
 *    Default: give default value.
 * </LI>
 * <LI> 
 *    ...
 * </LI>
 * </UL>
 *
 * <P><STRONG>POSITION IN STACK:</STRONG>
 *  <UL>
 *  <LI> 
 *    Describe how to embed that object into a protocol stack.
 *  </LI>
 *  </UL>
 *
 * <P><STRONG>USAGE:</STRONG>
 *  <UL>
 *  <LI> 
 *    If necessary provide usage information, such as types of URLs,
 *    possible errors, error management, etc.
 *  </LI>
 *  </UL>
 * 
 * <P><STRONG>DESIGN:</STRONG>
 *  <UL>
 *  <LI>
 *    Describe design. Provide class diagram and other documentation.
 *  </LI>
 *  </UL>
 *
 * <P><STRONG>EVENTS PRODUCED:</STRONG>
 *  <UL>
 *  <LI>
 *    list events produced.
 *  </LI>
 *  </UL>
 *
 * <P><STRONG>EVENTS HANDLED:</STRONG>
 *  <UL>
 *  <LI>
 *    lists events handled in this.upHandleEvent.
 *  </LI>
 *  </UL>
 *
 * <P><STRONG>EVENTS CONSUMED:</STRONG>
 *  <UL>
 *  <LI>
 *    lists events consumed in this.upHandleEvent.
 *  </LI>
 *  </UL>
 *
 * <P><STRONG>THREADS:</STRONG>
 *  <UL>
 *  <LI>
 *    Describe any threads that are spawned, synchronization issues,
 *    deadlock scenarios, etc.
 *  </LI>
 *  </UL>
 */
public class TEMPLATE extends ProtocolObject {
    /**
     * Create a TEMPLATE protocol object
     */
    public TEMPLATE()
    {
	super ("TEMPLATE");
    }
    
    /**
     * Initialize the protocol object after the protocol stack is
     * created. Also checks whether the right parameters were given to the 
     * protocol object.
     */
    public synchronized void dnInit()
    {
	Integer intVal;
	int testparam;

	// this object understands the following parameters:
	checkNameValueList("testparam");

	// process object parameters:
	if ((intVal = getValueAsInt("testparam")) != null) {
	    log_.info("dnInit: setting testparam to " 
		      + intVal.intValue());
	    if (intVal.intValue() < 0) {
		log_.panic("dnInit: testparam must be > 0");
	    }
	    testparam = intVal.intValue();
	}

	// pass this event down the stack:
	if (below() != null) below().dnInit();
    }

    /**
     * Flush the protocol object. Deliver any buffered messages, call
     * flush on any output streams.
     */
    public synchronized void dnFlush()
    {
	// pass this event down the stack:
	if (below() != null) below().dnFlush();
    }
    
    /**
     * Called by the object above to send a message to a channel by
     * multicast or unicast communication, depending on the channel URL:
     *
     * @param channel the destination channel of the posting
     * @param msg the message to be sent
     * @exception CommException on communication failure
     * @exception NotRegistered in case registerTalker was not called for
     *            the channel
     */
    public void dnPush(iBusURL channel, MessageEvent msg)
      throws CommException, NotRegistered
      {
	  synchronized (this) {
	      // NOTE: don't call dnPush from within a synchronized block.
	  }
	  // pass this event down the stack:
	  if (below() != null) below().dnPush(channel, msg);
      }

    /**
     * Called by the object above to subscribe to a channel.
     *
     * @param channel the channel to subscribe to
     * @exception AlreadySubscribed if this stack is already 
     *            subscribed to the channel
     * @exception CommException in case of a communication
     *            error
     */
    public synchronized void dnSubscribe(iBusURL channel)
      throws AlreadySubscribed, CommException
      {
	  // pass this event down the stack:
	  if (below() != null) below().dnSubscribe(channel);
      }
    
    /**
     * Called by the object above or by an iBus application to unsubscribe
     * from a certain channel.
     *
     * @param channel the channel to unsubscribe from
     * @exception NotSubscribed if this stack is not subscribed 
     *            to the channel
     * @exception CommException in case of a communication
     *            error
     */
    public synchronized void dnUnsubscribe(iBusURL channel)
      throws NotSubscribed, CommException
      {
	  // pass this event down the stack:
	  if (below() != null) below().dnUnsubscribe(channel);
      }

    /**
     * Called by the object above or by an iBus application to register
     * as a talker for a channel. 
     *
     * @param channel the channel to register for
     * @exception AlreadyRegistered if this stack is already 
     *            registered for the channel
     * @exception CommException in case of a communication
     *            error
     */
    public synchronized void dnRegisterTalker(iBusURL channel)
      throws AlreadyRegistered, CommException
      {
	  // pass this event down the stack:
	  if (below() != null) below().dnRegisterTalker(channel);
      }

    /**
     * Called by the object above or by an iBus application to unregister
     * as a talker from a channel.
     *
     * @param channel the channel to unregister from
     * @exception NotRegistered if this stack is not 
     *            registered with the channel
     * @exception CommException in case of a communication
     *            error
     */
    public synchronized void dnUnregisterTalker(iBusURL channel)
      throws NotRegistered, CommException
      {
	  // pass this event down the stack:
	  if (below() != null) below().dnUnregisterTalker(channel);
      }

    /**
     * Called by the protocol object beneath to pass an event up to me.
     * An event can be the arrival of a posting, the detection of a failure,
     * an acknowledgement, etc.
     *
     * @param event determines the type of event
     */
    public void upHandleEvent(Event event)
    {
	switch(event.id()) {
	case Event.evMessage:
	    MessageEvent msg =(MessageEvent)event;
	    // ...
	    break;

	case Event.evHeartbeat:
	    HeartbeatEvent hb =(HeartbeatEvent)event;
	    // ...
	    break;

	default:
	    // ...
	    break;
	}

	// By default, upHandleEvent passes the event up to the object
	// above of it:
	if (above() != null) above().upHandleEvent(event);
    }

    // Log object:
    protected static final Log log_ = new Log("TEMPLATE", true);
}
